@@ -2,7 +2,7 @@
 
 """ Dynamically control fan speed according to hard drive temperature. """
 
-__version__ = "1.4.2"
+__version__ = "1.5.0"
 __author__ = "desbma"
 __license__ = "GPLv3"
 
@@ -106,31 +106,72 @@ class Drive(HotDevice):
     HDPARM_GET_MODEL_REGEX = re.compile(r"Model Number:\s*(.*)")
     HDDTEMP_SLEEPING_SUFFIX = ": drive is sleeping\n"
 
+    class TempProbingMethod(enum.Enum):
+
+        """ Method used to query drive temperature, depending on what the drive/system support. """
+
+        HDDTEMP_INVOCATION = enum.auto()
+        HDDTEMP_DAEMON = enum.auto()
+        HDPARM_INVOCATION = enum.auto()
+        SMARTCTL_ATTRIB_INVOCATION = enum.auto()
+        SMARTCTL_SCT_INVOCATION = enum.auto()
+        DRIVETEMP = enum.auto()
+
     def __init__(
         self,
         device_filepath: str,
-        hddtemp_daemon_port: int,
+        hddtemp_daemon_port: Optional[int],
         min_temp: float,
         max_temp: float,
         use_smartctl: bool,
     ):
         assert stat.S_ISBLK(os.stat(device_filepath).st_mode)
-        self.device_filepath = self.__class__.normalizeDrivePath(device_filepath)
-        self.stat_filepath = f"/sys/block/{os.path.basename(self.device_filepath)}/stat"
+        device_filepath = self.__class__.normalizeDrivePath(device_filepath)
+        if not self.__class__.isPartition(device_filepath):
+            self.device_filepath = device_filepath
+            self.stat_filepath = f"/sys/block/{os.path.basename(self.device_filepath)}/stat"
+        else:
+            self.device_filepath = self.__class__.getParentDevice(device_filepath)
+            self.stat_filepath = (
+                f"/sys/block/{os.path.basename(self.device_filepath)}/{os.path.basename(device_filepath)}/stat"
+            )
         self.hddtemp_daemon_port = hddtemp_daemon_port
+
         self.min_temp = min_temp
         self.max_temp = max_temp
+
         self.pretty_name = self.getPrettyName()
         self.logger = logging.getLogger(str(self))
-        self.supports_hitachi_temp_query = self.supportsHitachiTempQuery()
-        self.supports_sct_temp_query = self.supportsSctTempQuery()
-        self.use_smartctl = use_smartctl
+
+        self.drivetemp_input_filepath = self.getDrivetempInputFilepath()
+        if use_smartctl:
+            if self.supportsSctTempQuery():
+                self.temp_query_method = Drive.TempProbingMethod.SMARTCTL_SCT_INVOCATION
+            else:
+                self.temp_query_method = Drive.TempProbingMethod.SMARTCTL_ATTRIB_INVOCATION
+        elif self.drivetemp_input_filepath is not None:
+            self.temp_query_method = Drive.TempProbingMethod.DRIVETEMP
+        elif self.supportsHitachiTempQuery():
+            self.temp_query_method = Drive.TempProbingMethod.HDPARM_INVOCATION
+        elif self.hddtemp_daemon_port is not None:
+            self.temp_query_method = Drive.TempProbingMethod.HDDTEMP_DAEMON
+        else:
+            self.temp_query_method = Drive.TempProbingMethod.HDDTEMP_INVOCATION
+        self.logger.info(f"Will probe temperature with method {self.temp_query_method.name}")
+
         self.probe_lock = threading.Lock()
         self.probe_count = 0
         self.get_state_lock = threading.Lock()
         self.get_state_count = 0
+
         self.warned_smartctl_attrib_counters = False
 
+        if self.__class__.isPartition(device_filepath):
+            self.logger.warning(
+                f"{device_filepath!r} is a partition, "
+                f"parent device {self.device_filepath!r} will be used except for activity stats"
+            )
+
     def __str__(self):
         """ Return a pretty drive name. """
         return self.pretty_name
@@ -147,6 +188,25 @@ class Drive(HotDevice):
         model = model_match.group(1).strip()
         return f"{os.path.basename(self.device_filepath)} {model}"
 
+    def getDrivetempInputFilepath(self) -> Optional[str]:
+        """ Return sysfs drivetemp input filepath if supported, None instead. """
+        try:
+            sysfs_bus_dir = f"/sys/block/{os.path.basename(self.device_filepath)}"
+            sysfs_bus_dir = os.path.normpath(
+                os.path.join(os.path.dirname(sysfs_bus_dir), os.readlink(sysfs_bus_dir), "..", "..")
+            )
+            with os.scandir(f"{sysfs_bus_dir}/hwmon") as dir_it:
+                for entry in filter(operator.methodcaller("is_dir"), dir_it):
+                    hwmon_dir = entry.path
+                    with open(f"{hwmon_dir}/name", "rt") as f:
+                        name = f.read().rstrip()
+                    if name == "drivetemp":
+                        return f"{hwmon_dir}/temp1_input"
+        except FileNotFoundError:
+            pass
+        self.logger.info("Drive does not support native drivetemp temp query")
+        return None
+
     def supportsHitachiTempQuery(self) -> bool:
         """ Test if drive supports hdparm -H. """
         supported = True
@@ -186,7 +246,7 @@ class Drive(HotDevice):
 
     def supportsProbingWhileAsleep(self) -> bool:
         """ Return True if drive can be probed while asleep, without waking up, False instead. """
-        return (not self.use_smartctl) and (self.supports_hitachi_temp_query)
+        return self.temp_query_method in (Drive.TempProbingMethod.HDPARM_INVOCATION, Drive.TempProbingMethod.DRIVETEMP)
 
     def getState(self) -> DriveState:
         """ Get drive power state, as a DriveState enum. """
@@ -217,29 +277,24 @@ class Drive(HotDevice):
 
     def getTemperature(self) -> float:
         """ Get drive temperature in Celcius. """
+        methods = {
+            Drive.TempProbingMethod.HDDTEMP_INVOCATION: self.getTemperatureWithHddtempInvocation,
+            Drive.TempProbingMethod.HDDTEMP_DAEMON: self.getTemperatureWithHddtempDaemon,
+            Drive.TempProbingMethod.HDPARM_INVOCATION: self.getTemperatureWithHdparmInvocation,
+            Drive.TempProbingMethod.SMARTCTL_ATTRIB_INVOCATION: self.getTemperatureWithSmartctlAttribInvocation,
+            Drive.TempProbingMethod.SMARTCTL_SCT_INVOCATION: self.getTemperatureWithSmartctlSctInvocation,
+            Drive.TempProbingMethod.DRIVETEMP: self.getTemperatureWithDrivetemp,
+        }
         with self.probe_lock:
-            if self.use_smartctl:
-                if self.supports_sct_temp_query:
-                    temp = self.getTemperatureWithSmartctlSctInvocation()
-                else:
-                    temp = self.getTemperatureWithSmartctlAttribInvocation()
-
-            elif self.supports_hitachi_temp_query:
-                temp = self.getTemperatureWithHdparmInvocation()
-
-            elif self.hddtemp_daemon_port is not None:
-                try:
-                    temp = self.getTemperatureWithHddtempDaemon()
-                except HddtempDaemonQueryFailed:
-                    self.logger.warning(
-                        "Hddtemp daemon returned an error when querying temperature for this drive, "
-                        "falling back to hddtemp process invocation. "
-                        "If that happens often, you may need to raise the hddtemp daemon priority "
-                        "(see: https://github.com/desbma/hddfancontrol/issues/15#issuecomment-461405402)."
-                    )
-                    temp = self.getTemperatureWithHddtempInvocation()
-
-            else:
+            try:
+                temp = methods[self.temp_query_method]()
+            except HddtempDaemonQueryFailed:
+                self.logger.warning(
+                    "Hddtemp daemon returned an error when querying temperature for this drive, "
+                    "falling back to hddtemp process invocation. "
+                    "If that happens often, you may need to raise the hddtemp daemon priority "
+                    "(see: https://github.com/desbma/hddfancontrol/issues/15#issuecomment-461405402)."
+                )
                 temp = self.getTemperatureWithHddtempInvocation()
 
             self.probe_count += 1
@@ -346,6 +401,12 @@ class Drive(HotDevice):
         temp_line = next(filter(lambda x: x.lstrip().startswith("Current Temperature: "), output.splitlines()))
         return int(temp_line.split()[2])
 
+    def getTemperatureWithDrivetemp(self) -> int:
+        """ Get drive temperature in Celcius using drivetemp sysfs. """
+        assert self.drivetemp_input_filepath is not None
+        with open(self.drivetemp_input_filepath, "rt") as f:
+            return int(f.read().rstrip()) // 1000
+
     def spinDown(self) -> None:
         """ Spin down a drive, effectively setting it to DriveState.STANDBY state. """
         self.logger.info(f"Spinning down drive {self}")
@@ -391,26 +452,35 @@ class Drive(HotDevice):
 
         # only reads have been registered, it can be data reads or just our own temp/state probes showing up in
         # the counters
-        if self.use_smartctl:
-            if self.supports_sct_temp_query:
-                expected_read_io_delta = temp_probe_count * 4
-                expected_read_sectors_delta = temp_probe_count * 3
-            else:  # attrib
-                if not self.warned_smartctl_attrib_counters:
-                    self.logger.warning(
-                        "Your kernel version and the current method of temperature probing "
-                        "does not allow reliably keeping track of drive activity. "
-                        "Auto spin down will not work."
-                    )
-                    self.warned_smartctl_attrib_counters = True
-                return True
 
-        elif self.supports_hitachi_temp_query:
+        if self.temp_query_method is Drive.TempProbingMethod.SMARTCTL_SCT_INVOCATION:
+            expected_read_io_delta = temp_probe_count * 4
+            expected_read_sectors_delta = temp_probe_count * 3
+
+        elif self.temp_query_method is Drive.TempProbingMethod.SMARTCTL_ATTRIB_INVOCATION:
+            if not self.warned_smartctl_attrib_counters:
+                self.logger.warning(
+                    "Your kernel version and the current method of temperature probing "
+                    "does not allow reliably keeping track of drive activity. "
+                    "Auto spin down will not work."
+                )
+                self.warned_smartctl_attrib_counters = True
+            return True
+
+        elif self.temp_query_method is Drive.TempProbingMethod.DRIVETEMP:
+            expected_read_io_delta = 0
+            expected_read_sectors_delta = 0
+
+        elif self.temp_query_method is Drive.TempProbingMethod.HDPARM_INVOCATION:
             expected_read_io_delta = temp_probe_count
             expected_read_sectors_delta = 0
 
         else:
             # hddtemp
+            assert self.temp_query_method in (
+                Drive.TempProbingMethod.HDDTEMP_INVOCATION,
+                Drive.TempProbingMethod.HDDTEMP_DAEMON,
+            )
             expected_read_io_delta = temp_probe_count * 5
             expected_read_sectors_delta = temp_probe_count * 3
 
@@ -460,6 +530,22 @@ class Drive(HotDevice):
             r = path
         return os.path.abspath(r)
 
+    @staticmethod
+    def isPartition(device_filepath: str) -> bool:
+        """ Return True if device is a partition, False otherwise. """
+        # https://github.com/python/mypy/issues/4177
+        assert __class__.normalizeDrivePath(device_filepath) == device_filepath  # type: ignore
+        return os.path.isfile(f"/sys/class/block/{os.path.basename(device_filepath)}/partition")
+
+    @staticmethod
+    def getParentDevice(device_filepath: str) -> str:
+        """ Get parent device from a partition device filepath. """
+        # https://github.com/python/mypy/issues/4177
+        assert __class__.normalizeDrivePath(device_filepath) == device_filepath  # type: ignore
+        sysfs_dev = os.readlink(f"/sys/class/block/{os.path.basename(device_filepath)}")
+        parent_dev = os.path.basename(os.path.dirname(sysfs_dev))
+        return f"/dev/{parent_dev}"
+
 
 class CPU(HotDevice):
 
@@ -883,7 +969,7 @@ def main(  # noqa: C901
     cpu_temp_range: Tuple[float, float],
     interval_s: int,
     spin_down_time_s: int,
-    hddtemp_daemon_port: int,
+    hddtemp_daemon_port: Optional[int],
     use_smartctl: bool,
 ):
     """ Run main program logic, after handling command line specific stuff. """
@@ -1033,7 +1119,7 @@ def cl_main():  # noqa: C901
         nargs="+",
         dest="fan_start_value",
         help="""PWM value (0-255), at which the fan starts moving.
-                                  Use the -t parameter, or run pwmconfig to find this value.""",
+                Use the -t parameter, or run pwmconfig to find this value.""",
     )
     arg_parser.add_argument(
         "--pwm-stop-value",
@@ -1042,8 +1128,8 @@ def cl_main():  # noqa: C901
         nargs="+",
         dest="fan_stop_value",
         help="""PWM value (0-255), at which the fan stop moving.
-                                  Use the -t parameter, or run pwmconfig to find this value.
-                                  Often 20-40 lower than start speed.""",
+                Use the -t parameter, or run pwmconfig to find this value.
+                Often 20-40 lower than start speed.""",
     )
     arg_parser.add_argument(
         "--min-temp",
@@ -1065,8 +1151,8 @@ def cl_main():  # noqa: C901
         default=20,
         dest="min_fan_speed_prct",
         help="""Minimum percentage of full fan speed to set the fan to.
-                                  Never set to 0 unless you have other fans to cool down your system,
-                                  or a case specially designed for passive cooling.""",
+                Never set to 0 unless you have other fans to cool down your system,
+                or a case specially designed for passive cooling.""",
     )
     arg_parser.add_argument(
         "-i",
@@ -1082,10 +1168,10 @@ def cl_main():  # noqa: C901
         default=None,
         dest="cpu_probe_filepath",
         help="""Also control fan speed according to this CPU temperature probe.
-                                  (ie. /sys/devices/platform/coretemp.0/hwmon/hwmonX/tempY_input).
-                                  WARNING: This is experimental, only use for low TDP CPUs. You
-                                  may need to set a low value for -i/--interval parameter to react quickly to
-                                  sudden CPU temperature increase.""",
+               (ie. /sys/devices/platform/coretemp.0/hwmon/hwmonX/tempY_input).
+                WARNING: This is experimental, only use for low TDP CPUs. You
+                may need to set a low value for -i/--interval parameter to react quickly to
+                sudden CPU temperature increase.""",
     )
     arg_parser.add_argument(
         "--cpu-temp-range",
@@ -1093,7 +1179,7 @@ def cl_main():  # noqa: C901
         nargs=2,
         default=(None, None),
         help="""CPU temperature range, if CPU temp monitoring is enabled.
-                                  If missing, will be autodetected or use a default value.""",
+                If missing, will be autodetected or use a default value.""",
     )
     arg_parser.add_argument(
         "--spin-down-time",
@@ -1133,7 +1219,7 @@ def cl_main():  # noqa: C901
         default=False,
         dest="hddtemp_daemon",
         help="""Get drive temperature from hddtemp daemon instead of spawning
-                                  a new process each time temperature is probed""",
+                a new process each time temperature is probed""",
     )
     arg_parser.add_argument(
         "--hddtemp-daemon-port",
@@ -1146,7 +1232,7 @@ def cl_main():  # noqa: C901
         "--smartctl",
         action="store_true",
         default=False,
-        help="""Probe temperature using smartctl instead of hddtemp/hdparm (EXPERIMENTAL)""",
+        help="""Probe temperature using smartctl instead of hddtemp/hdparm/drivetemp (EXPERIMENTAL)""",
     )
     args = arg_parser.parse_args()
     if ((args.fan_start_value is not None) and (len(args.fan_pwm_filepath) != len(args.fan_start_value))) or (
